import os, sys
import yaml
from pathlib import Path
from rich import print
from rich.prompt import Confirm, Prompt

BF_DEFAULT = {
    'lager_yeast': {
        'brew_time': 6,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'pumpkin_seeds': 16, 'sugar': 1}
    },
    'tequila': {
        'brew_time': 10,
        'age_time': 2,
        'age_type': 'acacia',
        'distill': 2,
        'ingred': {'cactus': 16, 'wild yeast': 1}
    },
    'wild_yeast': {
        'brew_time': 7,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'wheat_seeds': 16, 'sugar': 1}
    },
    'espresso': {
        'brew_time': 2,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'acacia_leaves': 6}
    },
    'wheatbeer': {
        'brew_time': 4,
        'age_time': None,
        'age_type': 'any',
        'distill': None,
        'ingred': {'wheat': 6}
    },
    'pilsner': {
        'brew_time': 14,
        'age_time': 2,
        'age_type': 'oak',
        'distill': None,
        'ingred': {'wheat': 8, 'lager_yeast': 1, 'tall_grass': 2}
    },
    'red_wine': {
        'brew_time': 9,
        'age_time': 14,
        'age_type': 'spruce',
        'distill': None,
        'ingred': {'vines': 11}
    },
    'chardonnay': {
        'brew_time': 11,
        'age_time': 4,
        'age_type': 'oak',
        'distill': None,
        'ingred': {
            'vines': 11,
            'twisting_vines': 4,
            'brewers_yeast': 1,
            'oak_leaves': 4
        }
    },
    'cider': {
        'brew_time': 4,
        'age_time': 1,
        'age_type': 'any',
        'distill': None,
        'ingred': {'apples': 8}
    },
    'cherry_juice': {
        'brew_time': 2,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'oak_leaves': 12, 'sugar': 3}
    },
    'whiskey': {
        'brew_time': 15,
        'age_time': 3,
        'age_type': 'any',
        'distill': 4,
        'ingred': {'wheat': 9, 'brewers_yeast': 1}
    },
    'potato_vodka': {
        'brew_time': 8,
        'age_time': None,
        'age_type': None,
        'distill': 3,
        'ingred': {'potatoes': 18, 'wild_yeast': 1}
    },
    'sake': {
        'brew_time': 18,
        'age_time': 3,
        'age_type': 'any',
        'distill': None,
        'ingred': {'seagrass': 15, 'wild_yeast': 1}
    },
    'kefir': {
        'brew_time': 10,
        'age_time': 3,
        'age_type': 'any',
        'distill': None,
        'ingred': {'milk_buckets': 6, 'wild_yeast': 1}
    },
    'earl_grey': {
        'brew_time': 6,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'dark_oak_leaves': 12, 'peony': 1}
    },
    'vegetable_soup': {
        'brew_time': 25,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'potatoes': 3, 'carrots': 3, 'beetroots': 2}
    },
    'zombie': {
        'brew_time': 1,
        'age_time': None,
        'age_type': None,
        'distill': None,
        'ingred': {'rum': 3, 'rotten_flesh': 2}
    }
}

CONFIG_DEFAULT = {
    'custom sound file name' : None,
    'timer mode': 'interval',
}

def file_handle(brewt_path = None):
    if brewt_path is None:
        brewt_path = os.path.expanduser('~') + "/.config/brewt"

    try:
        if os.path.exists(brewt_path + "/brews.yaml") == False or os.path.exists(brewt_path + "/config.yaml") == False:
            resp = Prompt.ask(f"[red]brewt didn't find files at [white]{brewt_path}[/white]\nwould you like for it to create default files there now?", 
            choices = ["yes","quit"] )
            if resp == "yes":
                if os.path.isdir(brewt_path) == False: os.mkdir(brewt_path)
                with open(brewt_path + "/brews.yaml", "w") as f:
                    yaml.safe_dump(BF_DEFAULT, f)
                    print(f"[green]created {brewt_path}/brews.yaml file...")
                with open(brewt_path + "/config.yaml", "w") as f:
                    yaml.safe_dump(CONFIG_DEFAULT, f)
                    print(f"[green]created {brewt_path}/config.yaml file...")
            elif resp == "quit":
                sys.exit(0)
    #except Exception as e:
    #    print(f"[red]couldn't complete file creation process\n{e}")
    #    sys.exit(0)
    except ValueError:
        pass

def validate_bf(brew_data):
    NoneType = type(None)

    cat_err = dict()
    type_err = dict()
    no_cat_err = dict()
    v_cat = set(['brew_time','age_time','age_type',"distill","ingred"])
    set_comp = set()

    for k, v in brew_data.items():
        if isinstance(brew_data[k],NoneType):
            no_cat_err[k] = f"no data entered for {k}."
            continue
        for i in v:
            set_comp.add(i)

        # too many keys
        comp_out = set_comp.difference(v_cat)
        if len(comp_out) > 0:
            cat_err[k] = f"bad category name: {comp_out}"

        # not enough keys
        comp_out = v_cat.difference(set_comp)
        if len(comp_out) > 0:
            cat_err[k] = f"missing category name: {comp_out}"
        set_comp.clear()

        for i in v_cat:
            if not isinstance(brew_data[k]['brew_time'],(int, float)):
                type_err[k] = f"incorrect type for 'brew_time'."
            if not isinstance(brew_data[k]['age_time'],(int, float, NoneType)):
                type_err[k] = f"incorrect type for 'age_time'."
            if not isinstance(brew_data[k]['age_type'],(str,NoneType)):
                type_err[k] = f"incorrect type for 'age_type'."
            if not isinstance(brew_data[k]['distill'],(int,NoneType)):
                type_err[k] = f"incorrect type for 'distill'."
            if not isinstance(brew_data[k]['ingred'],(dict)):
                type_err[k] = f"incorrect type for 'ingred'."

    if len(cat_err) or len(no_cat_err) or len(type_err):
        print(f"\n[red]errors found in [/red]brews.yaml[red]:")
        [ print(f"[red]{i}[/red] -> [orange]{v}[/orange]") for i, v in cat_err.items() ]
        [ print(f"[red]{i}[/red] -> [orange]{v}[/orange]") for i, v in no_cat_err.items() ]
        [ print(f"[red]{i}[/red] -> [orange]{v}[/orange]") for i, v in type_err.items() ]
        print("\n")
        sys.exit()

class Brewt_Config:
    def __init__(self, brewt_path) -> None:
        self.brewt_path = brewt_path
        with open(brewt_path + "/config.yaml", 'r') as f:
            self.config = yaml.safe_load(f)
        self.update_config()

    def update_config(self) -> None:
        self.sound_path = self.config['custom sound file name']
        self.timer_mode = self.config['timer mode']

    def change_config(self, sound_path = "", timer_mode = "") -> None:
        if sound_path == "": sound_path = self.sound_path
        if timer_mode == "": timer_mode = self.timer_mode

        conf_keys = self.config.keys()
        new_vals = ( sound_path, timer_mode )
        self.config = dict(zip(conf_keys, new_vals))
        self.update_config()
        with open(self.brewt_path + "/config.yaml", 'w') as f:
            yaml.safe_dump(self.config, f)




#!/usr/bin/env python3
import os, sys
import yaml
import rapidfuzz
import argparse

from brewt.brew_class import Brew
from brewt.bf_man import *

from rich import print
from rich import box
from rich.prompt import Prompt, Confirm
from rich.columns import Columns
from rich.panel import Panel
from rich.table import Table

aparser = argparse.ArgumentParser()
aparser.add_argument("-p", "--path", help="specify a location for brewt's files", default=f"{os.path.expanduser('~') + '/.config/brewt'}")
args = aparser.parse_args()

# file ops

file_handle(args.path)
with open(args.path + "/brews.yaml", 'r') as f:
    BREW_DATA = yaml.safe_load(f)
validate_bf(BREW_DATA)
BREW_DATA_KEYS = sorted(BREW_DATA.keys())
brewt_config_file = Brewt_Config(args.path)

def main():

    print(pretty_recipe_list())

    print(f"[yellow][bold]omokami's brewtool[/bold][/yellow]\n"
    f"[green]your configs are at [/green][white]{args.path}[/white]\n"
    f"[green]your timer mode is set to [white]{brewt_config_file.timer_mode}[white].[/green]\n\n",
    f"[green]type 'q' start over.\n\n",
    f"[yellow bold]slash commands:[/yellow bold]\n",
    f"[yellow]kill active timer [white](/k timer_id)\n",
    f"[yellow]list timer ids [white](/t)\n",
    f"[yellow]pretty print all recipe cards [white](/p)\n",
    f"[yellow]change timer mode [white](/ch interval/constant)\n",
    f"[yellow]specify a custom finish sound [white](/sp sound_path)\n",
    f"[yellow]exit program [white](/q)[/white]\n\n")

    menu_loop()

def pretty_recipe_list():

    pretty_keys = []
    for i in BREW_DATA_KEYS:
        pretty_keys.append(f"[green]{i.replace('_',' ')}[/green]")
    columns = Columns(pretty_keys,equal=True, expand=True, padding=(1,1), title="[bold yellow]brew file[/bold yellow]\n", align="center")
   
    return Panel(Panel(columns,width=75, padding=(1,2,1,2)), expand=False, box=box.MINIMAL)

def pretty_brew_card(brew_obj: Brew):
    
    card = list()
    card.append(f"[blue]ingredients:[/blue]")

    for i, q in brew_obj.ingred.items():
        card.append(f"[green]{q} {i.replace('_',' ')}[/green]")

    card.append("")
    card.append(f"[blue]brew for [/blue][white]{brew_obj.brew_time}[/white] [blue]minutes.[/blue]")

    if brew_obj.distill != None:
        card.append(f"[blue]distill [white]{brew_obj.distill}[/white] times.[/blue]")
    if brew_obj.age_time != None:
        card.append(f"[blue]age for [/blue][white]{brew_obj.age_time} [blue]years.[/blue]")

    card_table = Table(title=f"[yellow bold]{brew_obj.name.replace('_',' ')}[/yellow bold]", box=None)

    for i in card:
            card_table.add_row(i)

    return Panel(Panel(card_table, padding=(1,1,1,1)), expand=False, padding=(0,0,0,0), box=box.MINIMAL)

def pretty_card_list():

    every_brew_obj = []
    for i in BREW_DATA_KEYS:
        every_brew_obj.append(pretty_brew_card(Brew(i,BREW_DATA)))
    columns = Columns(every_brew_obj,equal=True, expand=True, padding=(1,1), title="[bold yellow]brew file[/bold yellow]\n", align="center")
   
    return Panel(Panel(columns, padding=(1,2,1,2)), expand=True, box=box.MINIMAL)

def menu_loop():
    running_timers = dict()    
    
    try:
        while True:
            running_timers = timer_cleaner(running_timers)

            # input handling
            brew_name = Prompt.ask("[yellow]brew name")
            if len(brew_name) == 0: continue
            if brew_name[0] == "/": 
                running_timers = slash_handle(brew_name,running_timers)
                continue
            if brew_name == "q": 
                continue

            # create brew objects if they match a config entry
            validated_brew = name_check(brew_name)
            if isinstance(validated_brew, str):
                new_brew = Brew(validated_brew, BREW_DATA, sound_path = brewt_config_file.sound_path)
                print()
            else: continue
            print(pretty_brew_card(new_brew))

            # confirm selection
            validate = Confirm.ask(f"[green]confirm [bold]{new_brew.name}[/bold]?",default="y")
            if not validate: continue

            # make calculations and add timers
            if new_brew.age_time != None:
                state = Prompt.ask("[yellow]state", choices=["b","brew","brewing","a","age","aging","q"], show_choices=False)
                if state == "q":
                    print("\n")
                    continue
                elif state == "b" or state =="brew" or state == "brewing":
                    state = "brew"
                else: 
                    state = "age"
            else:
                state = "brew"

            new_brew.start_timer(state, brewt_config_file.timer_mode)
            print(f"[yellow]brew created ([white]{new_brew.notif_code}[/white])")
            running_timers[new_brew.notif_code] = new_brew

    except KeyboardInterrupt:
        print("\n")
        sys.exit(0)

def slash_handle(query, running_timers):
    running_timers = timer_cleaner(running_timers)
    command = query[1:]
    
    if command[0] == "p":
        print(pretty_card_list())

    # kills a thread and pops it out of the active threads list
    elif command[0:2] == "k ":
        notif_code = command[2:len(command)]
        try:
            running_timers[notif_code].timer.terminate()
            running_timers.pop(notif_code)
        except KeyError:
            print(f"[white]{notif_code}[red] doesn't match any timers!")
            return running_timers
        print(f"[green]successfully terminated [white]{notif_code}")

    # lists all tids of running timers
    elif command[0] == "t":
        timer_data = set()
        for i in running_timers.values():
            if i.time_remaining.value != 0:
                timer_data.add(f"[green]{i.notif_code} has [white]{i.time_remaining.value}[/white] seconds left")
        if timer_data == set():
            print("[yellow bold]no timers currently active...")
        else:
            [ print(i) for i in timer_data ]
            timer_data.clear()

    # terminate all threads and then quit the program
    elif command[0] == "q":
        if running_timers is not None and len(running_timers) > 0:
            [ (i.timer.terminate(),print(f"[white]{i.notif_code}[/white][red] was terminated.")) for i in running_timers.values() ]
        os._exit(0)
    
    # change timer type
    elif command[0:3] == "ch ":
        new_val = command[3:len(command)]
        if new_val == "interval" or new_val == "constant":
            brewt_config_file.change_config(timer_mode = new_val)
            brewt_config_file.update_config()
            print(f"[green]updated config.yaml!")
        else: print(f"[red]invalid command!")

    # custom sound path
    elif command[0:3] == "sp ":
        new_val = command[3:len(command)]
        if os.path.exists(new_val):
            brewt_config_file.change_config(sound_path=new_val)
            brewt_config_file.update_config()
            print(f"[green]updated sound file!")
        else: print(f"[red]path {new_val} doesn't exist!")
    
    else:
        print(f"/{command} doesn't match any commands.")

    return running_timers

def name_check(name):
    # fuzzy matching for brew names
    name_search = rapidfuzz.process.extract(name, BREW_DATA_KEYS, limit=1)
    if name_search[0][1] < 50:
        print("no adequate match found for that brew...")
        return 0
    else:
        return name_search[0][0]

def timer_cleaner(running_timers):
    # check for done brews, use copy to avoid RunTimeError
    for k, v in running_timers.copy().items():
        if v.time_remaining.value < 1:
            running_timers.pop(k)
    return running_timers

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages 
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="brewt",
    version="1.2.0",
    description="a brewing tool for CivMC/CivCraft",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="omokami",
    url="https://github.com/okosuno/brewt",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: End Users/Desktop",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Topic :: Utilities"
    ],
    keywords="civcraft, civmc",
    python_requires=">=3.8",
    install_requires=["rich", "rapidfuzz", "playsound"],
    entry_points={
        "console_scripts": [
            "brewt=brewt.brewt:main",
        ]
    },
    project_urls={
    }
)



import os, time, datetime, math, dbus, uuid, playsound, multiprocessing
from multiprocessing import Value

class Brew:

    def __init__(self, name, brew_data, countdown_int = 5, sound_path = ""):
        self.name = name
        self.id = int(uuid.uuid4().fields[0])
        self.brew_time = brew_data[self.name]["brew_time"]
        self.age_time = brew_data[self.name]["age_time"]
        self.age_type = brew_data[self.name]["age_type"]
        self.distill = brew_data[self.name]["distill"]
        self.ingred = brew_data[self.name]["ingred"]
        self.name = self.name.replace("_"," ")
        self.notif_code = self.name[0:3] + str(self.id)[-4:-1]
        self.countdown_int = countdown_int
        self.sound_path = sound_path
        # shared memory value for reporting to /t
        self.time_remaining = Value('i', 0)

    def notifier(self, summ, msg, acts, hints, exp_time) -> None:
        # notification system info https://dbus.freedesktop.org/doc/dbus-python/
        # https://specifications.freedesktop.org/notification-spec/notification-spec-latest.html
        item = "org.freedesktop.Notifications"
        notify_int = dbus.Interface(
            dbus.SessionBus().get_object(item, "/"+item.replace(".", "/")), item)
        notify_int.Notify("brewt", f"{self.id}", u"🍶", summ, msg, acts, hints, exp_time)

    def check_state(self, state) -> tuple:
        # state checks
        if state == "brew":
            total_time = self.brew_time * 60
            state_str = f"remaining for brewing."
            end_str = "now "
            if self.distill is not None:
                end_str = end_str + f"distill {self.distill} times, then "
            if self.age_time is not None:
                end_str = end_str + f"age {self.name} for {self.age_time} years in {self.age_type} wood barrel."
            else:
                end_str = f"brewing is now complete!"
        elif state == "age":
            total_time = self.age_time * 60 * 20
            state_str = f"remaining for aging."
            end_str = f"aging is now complete!"
        else:
            return (None, None)
        return (state_str, end_str, total_time)

    def notif_timer_constant(self, state, time_remaining) -> None:
        state_str, end_str, total_time = self.check_state(state)
            
        # countdown before starting brew
        time_remaining.value = total_time
        time_remaining_pre = 5
        for s in range(0, self.countdown_int):
            self.notifier(f"{self.name}", f"starting timer in {time_remaining_pre} seconds...", [], {"urgency": 1}, 0)
            time_remaining_pre = 5 - (s + 1)
        for s in range(0, total_time):
            # drift protection appropriated from https://codereview.stackexchange.com/questions/199743/countdown-timer-in-python
            target= datetime.datetime.now()
            one_second_later = datetime.timedelta(seconds=1)

            # format time remaining string
            if time_remaining.value >= 60:
                hours_str = ""
                days_str = ""
                minutes = math.floor(time_remaining.value/ 60)
                seconds= time_remaining.value % 60
                if minutes > 59: 
                    hours = math.floor(minutes / 60)
                    hours_str = f"{hours}h "
                    minutes = minutes % 60
                    if hours > 24:
                        days = math.floor(hours / 24)
                        days_str = f"{days}d "
                formatted_time = f"{days_str}{hours_str}{minutes}m {seconds}s"
            else:
                formatted_time = time_remaining.value

            # send notification each second  
            self.notifier(f"{self.name}", f"{formatted_time} {state_str}", [], {"urgency": 1}, 0)
            target += one_second_later
            time.sleep((target - datetime.datetime.now()).total_seconds())
            time_remaining.value = total_time - (s + 1)

        # end notification lives for one minute
        self.notifier(f"{self.name}", f"{end_str}", [], {"urgency": 1}, 60000)

        # play end sound
        if self.sound_path == "":
            print('\a', end="")
        else:
            playsound.playsound(self.sound_path)

    def notif_timer_interval(self, state, time_remaining):
        state_str, end_str, total_time = self.check_state(state)
        interval_times = ( total_time, round(total_time / 2), round(total_time / 4), 60, 30, 5)

        # countdown before starting brew
        time_remaining.value = total_time
        time_remaining_pre = 5
        for s in range(0, self.countdown_int):
            self.notifier(f"{self.name}", f"starting timer in {time_remaining_pre} seconds...", [], {"urgency": 1}, 0)
            time_remaining_pre = 5 - (s + 1)
        
        for s in range(0, total_time):
            # drift protection appropriated from https://codereview.stackexchange.com/questions/199743/countdown-timer-in-python
            target= datetime.datetime.now()
            one_second_later = datetime.timedelta(seconds=1)

            # format time remaining string
            if time_remaining.value >= 60:
                hours_str = ""
                days_str = ""
                minutes = math.floor(time_remaining.value/ 60)
                seconds= time_remaining.value % 60
                if minutes > 59: 
                    hours = math.floor(minutes / 60)
                    hours_str = f"{hours}h "
                    minutes = minutes % 60
                    if hours > 24:
                        days = math.floor(hours / 24)
                        days_str = f"{days}d "
                formatted_time = f"{days_str}{hours_str}{minutes}m {seconds}s"
            else:
                formatted_time = f"{time_remaining.value}s"

            # send notification for each interval 
            for interval in interval_times:
                if time_remaining.value == interval:
                    self.notifier(f"{self.name} ({self.notif_code})", f"{formatted_time} {state_str}", [], {"urgency": 1}, 10000)
            target += one_second_later
            time.sleep((target - datetime.datetime.now()).total_seconds())
            time_remaining.value = total_time - (s + 1)

        # end notification lives for one minute
        self.notifier(f"{self.name}", f"{end_str}", [], {"urgency": 1}, 60000)

        # play end sound
        if self.sound_path == "":
            print('\a', end="")
        else:
            playsound.playsound(self.sound_path)

        

    def start_timer(self, state, type) -> None:
        if type == "interval": tar = self.notif_timer_interval
        elif type == "constant": tar = self.notif_timer_constant
        else: raise Exception("no valid timer type with which to start timer. delete your config.yaml and start fresh.")

        self.timer = multiprocessing.Process(target=tar, args=(state, self.time_remaining))
        self.timer.start()
